/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.iwg;

import de.hendriklipka.iwg.data.ClassDescriptor;
import de.hendriklipka.iwg.data.ParameterDescriptor;
import de.hendriklipka.iwg.data.MethodDescriptor;

import java.lang.reflect.*;

public class InterfaceScanner
{
    protected IBodyGenerator _bodyGen=null;

    public InterfaceScanner(IBodyGenerator bodyGen)
    {
        _bodyGen=bodyGen;
    }

    public void addInterfaceToClass(String interfaceName, ClassDescriptor classDesc, boolean implementProtected) throws ClassNotFoundException
    {
        classDesc.addInterface(interfaceName);
        createInterfaceMethods(interfaceName,classDesc,implementProtected);
    }

    protected void createInterfaceMethods(String interfaceName, ClassDescriptor classDesc, boolean implementProtected) throws ClassNotFoundException
    {
        try
        {
            Class iClass=Class.forName(interfaceName);
            System.out.println("INFO: implementing "+interfaceName);
            if (!iClass.isInterface())
            {
                System.err.println("ERROR: "+interfaceName+" is not an interface!");
                return;
            }
            String pName=iClass.getPackage().getName();
            String oName=_bodyGen.createFieldName(iClass.getName());
            oName=classDesc.addField(oName,iClass.getName());
            Method[] methods=iClass.getMethods();
            for (int i=0;i<methods.length;i++)
            {
                Method m=methods[i];
                int mod=m.getModifiers();
                if (Modifier.isPublic(mod))
                {
                    createMethod(m,oName,classDesc,interfaceName);
                }
                else if (Modifier.isProtected(mod)&&implementProtected)
                {
                    createMethod(m,oName,classDesc,interfaceName);
                }
            }
        }
        catch (ClassNotFoundException ex)
        {
                ex.printStackTrace();
        }
    }

    protected void createMethod(Method method, String objectName, ClassDescriptor classDesc, String interfaceName)
    {
        MethodDescriptor methodDesc=new MethodDescriptor();
        boolean isVoid=true;

        methodDesc._origin=interfaceName;
        // name
        methodDesc._name=method.getName();

        // return type (if not void)
        Class ret=method.getReturnType();
        if (ret!=Void.TYPE)
        {
            if (ret.isArray())
            {
                methodDesc._returnType=ret.getComponentType().getName()+"[]";
            }
            else
            {
                methodDesc._returnType=ret.getName();
            }
        }

        // public / protected
        int mod=method.getModifiers();
        if (Modifier.isPublic(mod))
        {
            methodDesc._isPublic=true;
        }
        else if (Modifier.isProtected(mod))
        {
            methodDesc._isPublic=false;
        }
        if (Modifier.isSynchronized(mod))
        {
            methodDesc._isSynchronized=true;
        }

        // parameters
        Class[] paramClasses=method.getParameterTypes();
        methodDesc._params=new ParameterDescriptor[paramClasses.length];
        for (int i=0;i<paramClasses.length;i++)
        {
            ParameterDescriptor pd=new ParameterDescriptor();
            if (paramClasses[i].isArray())
            {
                pd._type=paramClasses[i].getComponentType().getName()+"[]";
            }
            else
            {
                pd._type=paramClasses[i].getName();
            }
            pd._name="p"+i;
            methodDesc._params[i]=pd;
        }

        // exceptions
        Class[] exa=method.getExceptionTypes();
        methodDesc._exceptions=new String[exa.length];
        for (int i = 0; i < exa.length; i++)
        {
            Class ex = exa[i];
            methodDesc._exceptions[i]=ex.getName();
        }

        // field name
        methodDesc._fieldName=objectName;

        // add to class
        classDesc.addMethod(methodDesc);
    }
}
